/*
 *
 * Copyright 1998-1999, University of Notre Dame.
 * Authors: Jeffrey M. Squyres, Kinis L. Meyer with M. D. McNally 
 *          and Andrew Lumsdaine
 *
 * This file is part of the Notre Dame LAM implementation of MPI.
 *
 * You should have received a copy of the License Agreement for the
 * Notre Dame LAM implementation of MPI along with the software; see
 * the file LICENSE.  If not, contact Office of Research, University
 * of Notre Dame, Notre Dame, IN 46556.
 *
 * Permission to modify the code and to distribute modified code is
 * granted, provided the text of this NOTICE is retained, a notice that
 * the code was modified is included with the above COPYRIGHT NOTICE and
 * with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
 * file is distributed with the modified code.
 *
 * LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
 * By way of example, but not limitation, Licensor MAKES NO
 * REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
 * PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
 * OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
 * OR OTHER RIGHTS.  
 *
 * Additional copyrights may follow.
 *
 *	Ohio Trollius
 *	Copyright 1997 The Ohio State University
 *	NJN
 *
 *	$Id: nameunpub.c,v 6.3 1999/05/25 22:08:09 kmeyer1 Exp $
 *
 *	Function:	- unpublish a port/service pair
 *	Accepts:	- service name
 *			- info
 *			- port name
 *	Returns:	- MPI_SUCCESS or error code
 */

#include <errno.h>

#include <blktype.h>
#include <mpi.h>
#include <mpisys.h>


int
MPI_Unpublish_name(service_name, info, port_name)

char			*service_name;
MPI_Info		info;
char			*port_name;

{
	lam_initerr();
	lam_setfunc(BLKMPIUNPUBLISHNAME);
/*
 * Check arguments.
 */
	if (service_name == 0) {
		return(lam_errfunc(MPI_COMM_WORLD, BLKMPIUNPUBLISHNAME,
					lam_mkerr(MPI_ERR_ARG, 0)));
	}
/*
 * We do not allow multiple ports for the same service name and hence
 * ignore the port name argument.
 */
	if (lam_unpublish(service_name)) {
		return(lam_errfunc(MPI_COMM_WORLD, BLKMPIUNPUBLISHNAME,
				lam_mkerr(MPI_ERR_SERVICE, errno))); 
	}

        lam_resetfunc(BLKMPIUNPUBLISHNAME);
	return(MPI_SUCCESS);
}
