/*
 *
 * Copyright 1998-1999, University of Notre Dame.
 * Authors: Jeffrey M. Squyres, Kinis L. Meyer with M. D. McNally 
 *          and Andrew Lumsdaine
 *
 * This file is part of the Notre Dame LAM implementation of MPI.
 *
 * You should have received a copy of the License Agreement for the
 * Notre Dame LAM implementation of MPI along with the software; see
 * the file LICENSE.  If not, contact Office of Research, University
 * of Notre Dame, Notre Dame, IN 46556.
 *
 * Permission to modify the code and to distribute modified code is
 * granted, provided the text of this NOTICE is retained, a notice that
 * the code was modified is included with the above COPYRIGHT NOTICE and
 * with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
 * file is distributed with the modified code.
 *
 * LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
 * By way of example, but not limitation, Licensor MAKES NO
 * REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
 * PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
 * OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
 * OR OTHER RIGHTS.  
 *
 * Additional copyrights may follow.
 *
 *	Ohio Trollius
 *	Copyright 1996 The Ohio State University
 *	NJN
 *
 *	$Id: portclose.c,v 6.3 1999/05/25 22:08:11 kmeyer1 Exp $
 *
 *	Function:	- close an MPI port
 *	Accepts:	- port name
 *	Returns:	- MPI_SUCCESS or error code
 */

#include <string.h>

#include <blktype.h>
#include <mpi.h>
#include <mpisys.h>
#include <terror.h>


int
MPI_Close_port(port_name)

char			*port_name;

{
	struct _port	search;			/* search key for port */
	struct _port	*port;			/* found port */

	lam_initerr();
	lam_setfunc(BLKMPICLOSEPORT);

	if (strlen(port_name) > LAM_PORTLEN) {
		return(lam_errfunc(MPI_COMM_WORLD, BLKMPICLOSEPORT,
					lam_mkerr(MPI_ERR_ARG, 0)));
	}
/*
 * Find the port.
 */
	strncpy(search.prt_name, port_name, LAM_PORTLEN);

	port = al_find(lam_ports, &search);
	if (port == 0) {
		return(lam_errfunc(MPI_COMM_WORLD, BLKMPICLOSEPORT,
					lam_mkerr(MPI_ERR_OTHER, ENOENT)));
	}
/*
 * close it
 */
	if (lam_port_close(port->prt_num)) {
		return(lam_errfunc(MPI_COMM_WORLD, BLKMPICLOSEPORT,
					lam_mkerr(MPI_ERR_OTHER, errno)));
	}
	if (al_delete(lam_ports, port)) {
		return(lam_errfunc(MPI_COMM_WORLD, BLKMPICLOSEPORT,
					lam_mkerr(MPI_ERR_INTERN, 0)));
	}

        lam_resetfunc(BLKMPICLOSEPORT);
	return(MPI_SUCCESS);
}
