/*
 *
 * Copyright 1998-1999, University of Notre Dame.
 * Authors: Jeffrey M. Squyres, Kinis L. Meyer with M. D. McNally 
 *          and Andrew Lumsdaine
 *
 * This file is part of the Notre Dame LAM implementation of MPI.
 *
 * You should have received a copy of the License Agreement for the
 * Notre Dame LAM implementation of MPI along with the software; see
 * the file LICENSE.  If not, contact Office of Research, University
 * of Notre Dame, Notre Dame, IN 46556.
 *
 * Permission to modify the code and to distribute modified code is
 * granted, provided the text of this NOTICE is retained, a notice that
 * the code was modified is included with the above COPYRIGHT NOTICE and
 * with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
 * file is distributed with the modified code.
 *
 * LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
 * By way of example, but not limitation, Licensor MAKES NO
 * REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
 * PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
 * OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
 * OR OTHER RIGHTS.  
 *
 * Additional copyrights may follow.
 *
 *	Ohio Trollius
 *	Copyright 1997 The Ohio State University
 *	RBD/GDB/NJN
 *
 *	$Id: testall.c,v 6.3 1999/05/25 22:08:20 kmeyer1 Exp $
 *
 *	Function:	- check if all requests completed
 *	Accepts:	- # of requests
 *			- array of requests
 *			- flag (out)
 *			- array of status structures (modified)
 *	Returns:	- MPI_SUCCESS or error code
 */

#include <blktype.h>
#include <mpi.h>
#include <mpisys.h>


int
MPI_Testall(count, reqs, flag, stats)

int			count;
MPI_Request		*reqs;
int			*flag;
MPI_Status		*stats;

{
	int		i;			/* favourite index */
	int		fl_done;		/* request done? */
	int		err;			/* error code */
	int		fl_trace;		/* do tracing? */
	MPI_Request	*req;			/* ptr request */
	MPI_Status	*stat;			/* ptr status */

	lam_initerr_m();
	lam_setfunc_m(BLKMPITESTALL);

	if (count < 0) {
		return(lam_errfunc(MPI_COMM_WORLD, BLKMPITESTALL,
				lam_mkerr(MPI_ERR_COUNT, 0)));
	}

	if ((count > 0) && (reqs == 0)) {
		return(lam_errfunc(MPI_COMM_WORLD, BLKMPITESTALL,
				lam_mkerr(MPI_ERR_ARG, 0)));
	}

	if (flag == 0) {
		return(lam_errfunc(MPI_COMM_WORLD, BLKMPITESTALL,
				lam_mkerr(MPI_ERR_ARG, 0)));
	}
/*
 * Generate a start wrapper trace.
 */
	fl_trace = ((_kio.ki_rtf & RTF_TRON) == RTF_TRON);

	if (fl_trace) {
		lam_tr_wrapstart(BLKMPITESTALL);
	}
/*
 * Initialize all status members.
 */
	if (stats != MPI_STATUSES_IGNORE) {
		for (i = 0, stat = stats; i < count; ++i, ++stat) {
			lam_emptystat(stat);
		}
	}
/*
 * Loop testing all requests non-destructively.
 * Stop at the first request not done.
 */
	req = reqs;
	stat = stats;

	for (i = 0; i < count; ++i, ++req) {

		err = lam_test(req, 0, &fl_done, stat);

		if (err != MPI_SUCCESS) {
			stat->MPI_ERROR = err;

			if (fl_trace) {
				lam_tr_wrapend(BLKMPITESTALL);
			}

			return(lam_errfunc(MPI_COMM_WORLD, BLKMPITESTALL,
					lam_mkerr(MPI_ERR_IN_STATUS, 0)));
		}

		if (fl_done == 0) {
			*flag = 0;

			if (fl_trace) {
				lam_tr_wrapend(BLKMPITESTALL);
			}

			lam_resetfunc_m(BLKMPITESTALL);
			return(MPI_SUCCESS);
		}

		if (stats != MPI_STATUSES_IGNORE) {
			stat++;
		}
	}
/*
 * All active requests are done, loop doing the job.
 */
	req = reqs;
	stat = stats;

	for (i = 0; i < count; ++i, ++req) {
		err = lam_test(req, 1, &fl_done, stat);

		if (err != MPI_SUCCESS) {
			stat->MPI_ERROR = err;

			if (fl_trace) {
				lam_tr_wrapend(BLKMPITESTALL);
			}

			return(lam_errfunc(MPI_COMM_WORLD, BLKMPITESTALL,
					lam_mkerr(MPI_ERR_IN_STATUS, 0)));
		}

		if (stats != MPI_STATUSES_IGNORE) {
			stat++;
		}
	}

	*flag = 1;
/*
 * Generate an end wrapper trace.
 */
	if (fl_trace) {
		lam_tr_wrapend(BLKMPITESTALL);
	}

	lam_resetfunc_m(BLKMPITESTALL);
	return(MPI_SUCCESS);
}
