/*
 *
 * Copyright 1998-1999, University of Notre Dame.
 * Authors: Jeffrey M. Squyres, Kinis L. Meyer with M. D. McNally 
 *          and Andrew Lumsdaine
 *
 * This file is part of the Notre Dame LAM implementation of MPI.
 *
 * You should have received a copy of the License Agreement for the
 * Notre Dame LAM implementation of MPI along with the software; see
 * the file LICENSE.  If not, contact Office of Research, University
 * of Notre Dame, Notre Dame, IN 46556.
 *
 * Permission to modify the code and to distribute modified code is
 * granted, provided the text of this NOTICE is retained, a notice that
 * the code was modified is included with the above COPYRIGHT NOTICE and
 * with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
 * file is distributed with the modified code.
 *
 * LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
 * By way of example, but not limitation, Licensor MAKES NO
 * REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
 * PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
 * OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
 * OR OTHER RIGHTS.  
 *
 * Additional copyrights may follow.
 *
 *	Ohio Trollius
 *	Copyright 1997 The Ohio State University
 *	RBD/GDB
 *
 *	$Id: testsome.c,v 6.3 1999/05/25 22:08:20 kmeyer1 Exp $
 *
 *	Function:	- check if at least one request completed
 *			- return info on all completed requests
 *	Accepts:	- # of requests
 *			- array of requests
 *			- output count (out)
 *			- array of indices (out)
 *			- array of status structures (out)
 *	Returns:	- MPI_SUCCESS or error code
 */

#include <blktype.h>
#include <mpi.h>
#include <mpisys.h>
#include <mpitrace.h>
#include <rpisys.h>


int
MPI_Testsome(count, reqs, outcount, indices, stats)

int			count;
MPI_Request		*reqs;
int			*outcount;
int			*indices;
MPI_Status		*stats;

{
	int		i;			/* favourite index */
	int		fl_done;		/* request done flag */
	int		fl_err;			/* error flag */
	int		err;			/* error code */
	int		fl_trace;		/* do tracing? */
	MPI_Status	*p;			/* ptr status */

	lam_initerr_m();
	lam_setfunc_m(BLKMPITESTSOME);
/*
 * Check the arguments.
 */
	if (count < 0) {
		return(lam_errfunc(MPI_COMM_WORLD, BLKMPITESTSOME,
				lam_mkerr(MPI_ERR_COUNT, 0)));
	}

	if ((count > 0) && ((reqs == 0) || (indices == 0))) {
		return(lam_errfunc(MPI_COMM_WORLD, BLKMPITESTSOME,
				lam_mkerr(MPI_ERR_ARG, 0)));
	}

	if (outcount == 0) {
		return(lam_errfunc(MPI_COMM_WORLD, BLKMPITESTSOME,
				lam_mkerr(MPI_ERR_ARG, 0)));
	}
/*
 * Generate a start wrapper trace.
 */
	fl_trace = ((_kio.ki_rtf & RTF_TRON) == RTF_TRON);

	if (fl_trace) {
		lam_tr_wrapstart(BLKMPITESTSOME);
	}
/*
 * Initialize all status members.
 */
	if (stats != MPI_STATUSES_IGNORE) {
		for (i = 0, p = stats; i < count; ++i, ++p) {
			lam_emptystat(p);
		}
	}
/*
 * Loop testing all requests.
 * Skip inactive requests.
 */
	*outcount = MPI_UNDEFINED;
	fl_err = 0;

	for (i = 0; i < count; ++i, ++reqs) {

		if ((*reqs == MPI_REQUEST_NULL) ||
			((*reqs)->rq_state == LAM_RQSINIT)) continue;

		if (*outcount == MPI_UNDEFINED) {
			*outcount = 0;
		}

		err = MPI_Test(reqs, &fl_done, stats);

		if ((err != MPI_SUCCESS) || fl_done) {

			++(*outcount);
			*indices++ = i;

			if (err != MPI_SUCCESS) {
				if (stats != MPI_STATUSES_IGNORE) {
					stats->MPI_ERROR = err;
				}

				fl_err = 1;
			}

			if (stats != MPI_STATUSES_IGNORE) {
				++stats;
			}
		}
	}
/*
 * Generate an end wrapper trace.
 */
	if (fl_trace) {
		lam_tr_wrapend(BLKMPITESTSOME);
	}

	if (fl_err) {
		return(lam_errfunc(MPI_COMM_WORLD, BLKMPITESTSOME,
				lam_mkerr(MPI_ERR_IN_STATUS, 0)));
	}

	lam_resetfunc_m(BLKMPITESTSOME);
	return(MPI_SUCCESS);
}
