/*
 *
 * Copyright 1998-1999, University of Notre Dame.
 * Authors: Jeffrey M. Squyres, Kinis L. Meyer with M. D. McNally 
 *          and Andrew Lumsdaine
 *
 * This file is part of the Notre Dame LAM implementation of MPI.
 *
 * You should have received a copy of the License Agreement for the
 * Notre Dame LAM implementation of MPI along with the software; see
 * the file LICENSE.  If not, contact Office of Research, University
 * of Notre Dame, Notre Dame, IN 46556.
 *
 * Permission to modify the code and to distribute modified code is
 * granted, provided the text of this NOTICE is retained, a notice that
 * the code was modified is included with the above COPYRIGHT NOTICE and
 * with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
 * file is distributed with the modified code.
 *
 * LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
 * By way of example, but not limitation, Licensor MAKES NO
 * REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
 * PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
 * OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
 * OR OTHER RIGHTS.  
 *
 * Additional copyrights may follow.
 *
 *	Ohio Trollius
 *	Copyright 1995 The Ohio State University
 *	GDB/RBD
 *
 *	$Id: frecv.c,v 6.3 1999/09/10 05:22:20 jsquyres Exp $
 * 
 *	Function:	- forwarding datalink recv
 */

#include <net.h>

/*
 *	frecv
 *
 *	Function:	- receives a forwarded message at the datalink sublayer
 *			- the header is overwritten by the received message
 *	Accepts:	- network message descriptor
 *	Returns:	- 0 or ERROR
 */
int
frecv(nhead)

struct nmsg		*nhead;

{
	struct kmsg	khead;		/* kernel message descriptor */
/*
 * Set up kernel message descriptor to receive network frame header.
 */     
	LAM_ZERO_ME(khead);
	khead.k_event = nhead->nh_event;
	khead.k_type = nhead->nh_type;
	khead.k_length = nhead->nh_length;
	khead.k_flags = KNMSG | nhead->nh_flags;
	khead.k_msg = (char *) nhead;
/*
 * Receive network frame header.
 */
	return(krecv(&khead));
}

/*
 *	frecvfront
 *
 *	Function:	- front half of frecv()
 *	Accepts:	- network message descriptor
 *	Returns:	- client-kernel socket or ERROR
 */
int
frecvfront(nhead)

struct nmsg		*nhead;

{
	struct kmsg	khead;		/* kernel message descriptor */
/*
 * Set up kernel message descriptor to receive network frame header.
 */     
	khead.k_event = nhead->nh_event;
	khead.k_type = nhead->nh_type;
	khead.k_length = nhead->nh_length;
	khead.k_flags = KNMSG | nhead->nh_flags;
	khead.k_msg = (char *) nhead;
/*
 * Receive network frame header.
 */
	return(krecvfront(&khead));
}

/*
 *	frecvback
 *
 *	Function:	- back half of frecv()
 *	Accepts:	- network message descriptor
 *	Returns:	- 0 or ERROR
 */
int
frecvback(nhead)

struct nmsg		*nhead;

{
	struct kmsg	khead;		/* kernel message descriptor */

	khead.k_flags = KNMSG | nhead->nh_flags;
	khead.k_msg = (char *) nhead;
	return(krecvback(&khead));
}
